require 'sawyer'
require 'strife/configurable'
require 'strife/client/champion'
require 'strife/client/game'
require 'strife/client/league'
require 'strife/client/stats'
require 'strife/client/summoner'
require 'strife/client/team'

module Strife
  class Client
    include Strife::Configurable
    include Strife::Client::Champion
    include Strife::Client::Game
    include Strife::Client::League
    include Strife::Client::Stats
    include Strife::Client::Summoner
    include Strife::Client::Team

    def initialize(options = {})
      Strife::Configurable.keys.each do |key|
        instance_variable_set(:"@#{key}", options[key] || Strife.instance_variable_get(:"@#{key}"))
      end
    end

    def same_options?(opts)
      opts.hash == options.hash
    end

    # Scrub out our api key from #inspect
    def inspect
      inspected = super

      inspected = inspected.gsub! @api_key, '*' * @api_key.length

      inspected
    end

    def get(url, options = {})
      request :get, url, options
    end

    def last_response
      @last_response
    end

    def agent
      @agent ||= Sawyer::Agent.new("#{@api_endpoint}#{@region}/", sawyer_options) do |http|
        http.headers[:accept] = @default_media_type
        http.headers[:user_agent] = @user_agent
      end
    end

    private

    def sawyer_options
      opts = {
        links_parser: Sawyer::LinkParsers::Simple.new
      }

      conn_opts = @connection_options
      conn_opts[:builder] = @middleware if @middleware
      opts[:faraday] = Faraday.new(conn_opts)

      opts
    end

    def request(method, path, options)
      options[:query] ||= {}
      options[:query].merge!(api_key: @api_key)

      @last_response = response = agent.call(method, URI.encode(path.to_s), options)
      response.data
    end
  end
end
