module Strife
  module Configurable
    attr_accessor :api_key, :region
    attr_writer :api_endpoint

    class << self
      def keys
        @keys ||= [
          :api_key,
          :api_endpoint,
          :user_agent,
          :default_media_type,
          :connection_options,
          :middleware,
          :region
        ]
      end
    end

    def configure
      yield self
    end

    def reset!
      Strife::Configurable.keys.each do |key|
        instance_variable_set(:"@#{key}", Strife::Default.options[key])
      end

      self
    end
    alias setup reset!

    private

    def options
      Hash[Strife::Configurable.keys.map {|key| [key, instance_variable_get(:"@#{key}")]}]
    end
  end
end
