require 'strife/response/raise_error'
require 'faraday-http-cache'
require 'strife/version'

module Strife
  module Default
    API_ENDPOINT = 'http://prod.api.pvp.net/api/lol/'.freeze

    USER_AGENT = "Strife Riot API GEM v#{Strife::VERSION}".freeze

    MEDIA_TYPE = 'application/json'.freeze

    REGION = 'na'.freeze

    MIDDLEWARE = Faraday::Builder.new do |builder|
      builder.use Strife::Response::RaiseError
      builder.use :http_cache, store: :memory_store
      builder.adapter Faraday.default_adapter
    end

    # Default options
    class << self
      def options
        Hash[Strife::Configurable.keys.map {|key| [key, send(key)]}]
      end

      def api_key
        ENV['STRIFE_API_KEY']
      end

      def api_endpoint
        ENV['STRIFE_API_ENDPOINT'] || API_ENDPOINT
      end

      def user_agent
        ENV['STRIFE_USER_AGENT'] || USER_AGENT
      end

      def default_media_type
        ENV['STRIFE_DEFAULT_MEDIA_TYPE'] || MEDIA_TYPE
      end

      def middleware
        MIDDLEWARE
      end

      def connection_options
        {
          headers: {
            accept: default_media_type,
            user_agent: user_agent
          }
        }
      end

      def region
        ENV['STRIFE_REGION'] || REGION
      end
    end
  end
end
